import warnings

from rich_click.rich_command import RichGroup

warnings.warn(
    "RichCommand is moving from rich_click.rich_group to rich_click.rich_command in a future version.",
    DeprecationWarning,
)


__all__ = ["RichGroup"]



from typing import Any, IO, Optional

import click
import rich
import rich.highlighter
import rich.markdown
import rich.padding
import rich.text
import rich.theme
from rich.console import Console

from rich_click.rich_help_configuration import RichHelpConfiguration


def create_console(config: RichHelpConfiguration, file: Optional[IO[str]] = None) -> Console:
    """Create a Rich Console configured from Rich Help Configuration.

    Args:
        config: Rich Help Configuration instance
        file: Optional IO stream to write Rich Console output
            Defaults to None.
    """
    console = Console(
        theme=rich.theme.Theme(
            {
                "option": config.style_option,
                "command": config.style_command,
                "argument": config.style_argument,
                "switch": config.style_switch,
                "metavar": config.style_metavar,
                "metavar_sep": config.style_metavar_separator,
                "usage": config.style_usage,
            }
        ),
        highlighter=config.highlighter,
        color_system=config.color_system,
        force_terminal=config.force_terminal,
        file=file,
        width=config.width,
        legacy_windows=config.legacy_windows,
    )
    if isinstance(config.max_width, int):
        console.width = min(config.max_width, console.size.width)
    return console


def get_module_config() -> RichHelpConfiguration:
    """Get the module-level help configuration.

    A function-level import is used to avoid a circular dependency
    between the formatter and formatter operations.
    """
    from rich_click.rich_click import get_module_help_configuration

    return get_module_help_configuration()


class RichHelpFormatter(click.HelpFormatter):
    """Rich Help Formatter.

    This class is a container for the help configuration and Rich Console that
    are used internally by the help and error printing methods.
    """

    def __init__(
        self,
        indent_increment: int = 2,
        width: Optional[int] = None,
        max_width: Optional[int] = None,
        *args: Any,
        config: Optional[RichHelpConfiguration] = None,
        **kwargs: Any,
    ) -> None:
        """Create Rich Help Formatter.

        Args:
            config: Configuration.
                Defaults to None.
        """
        if config is not None:
            # Rich config overrides width and max width if set.
            width = config.width or width
            max_width = config.max_width or max_width
        super().__init__(indent_increment, width, max_width, *args, **kwargs)
        self._config = config or get_module_config()
        self._console = create_console(self._config)

    @property
    def config(self) -> RichHelpConfiguration:
        """Rich Help Configuration."""
        return self._config

    @property
    def console(self) -> Console:
        """Rich Console created from the help configuration.

        This console is meant only for use with the formatter and should
        not be created directly
        """
        return self._console

    def write(self, string: str) -> None:
        return self._console.print(string)



from typing import Any, Optional, Type

import click
from rich.console import Console

from rich_click.rich_help_configuration import RichHelpConfiguration
from rich_click.rich_help_formatter import RichHelpFormatter


class RichContext(click.Context):
    """Click Context class endowed with Rich superpowers."""

    formatter_class: Type[RichHelpFormatter] = RichHelpFormatter

    def __init__(
        self,
        *args: Any,
        rich_console: Optional[Console] = None,
        rich_help_config: Optional[RichHelpConfiguration] = None,
        **kwargs: Any,
    ) -> None:
        """Create Rich Context instance.

        Args:
            rich_console: Rich Console.
                Defaults to None.
            rich_help_config: Rich help configuration.
                Defaults to None.
        """
        super().__init__(*args, **kwargs)
        parent: Optional[RichContext] = kwargs.pop("parent", None)

        if rich_console is None and hasattr(parent, "console"):
            rich_console = parent.console  # type: ignore[has-type,union-attr]

        self.console = rich_console

        if rich_help_config is None and hasattr(parent, "help_config"):
            rich_help_config = parent.help_config  # type: ignore[has-type,union-attr]

        self.help_config = rich_help_config

    def make_formatter(self) -> RichHelpFormatter:
        """Create the Rich Help Formatter."""
        return self.formatter_class(
            width=self.terminal_width, max_width=self.max_content_width, config=self.help_config
        )



from typing import Any, Optional


def truthy(o: Any) -> Optional[bool]:
    """Check if string or other obj is truthy."""
    if isinstance(o, str):
        if o.lower() in {"y", "yes", "t", "true", "1"}:
            return True
        elif o.lower() in {"n", "no", "f", "false", "0"}:
            return False
        else:
            return None
    elif o is None:
        return None
    else:
        return bool(o)



import os
from dataclasses import dataclass, field
from os import getenv
from typing import Any, Dict, List, Optional, Tuple, Union

import rich.align
import rich.highlighter
import rich.padding
import rich.style
import rich.table
from typing_extensions import Literal

from rich_click.utils import truthy


def force_terminal_default() -> Optional[bool]:
    """Use as the default factory for `force_terminal`."""
    env_vars = {"GITHUB_ACTIONS", "FORCE_COLOR", "PY_COLORS"}
    if all(i not in os.environ for i in env_vars):
        return None
    else:
        return any(truthy(getenv(i)) for i in env_vars)


def terminal_width_default() -> Optional[int]:
    """Use as the default factory for `width` and `max_width`."""
    width = getenv("TERMINAL_WIDTH")
    if width:
        try:
            return int(width)
        except ValueError:
            import warnings

            warnings.warn("Environment variable `TERMINAL_WIDTH` cannot be cast to an integer.", UserWarning)
            return None
    return None


class OptionHighlighter(rich.highlighter.RegexHighlighter):
    """Highlights our special options."""

    highlights = [
        r"(^|\W)(?P<switch>\-\w+)(?![a-zA-Z0-9])",
        r"(^|\W)(?P<option>\-\-[\w\-]+)(?![a-zA-Z0-9])",
        r"(?P<metavar>\<[^\>]+\>)",
    ]


@dataclass
class RichHelpConfiguration:
    """Rich Help Configuration Class."""

    # Default styles
    style_option: rich.style.StyleType = field(default="bold cyan")
    style_argument: rich.style.StyleType = field(default="bold cyan")
    style_command: rich.style.StyleType = field(default="bold cyan")
    style_switch: rich.style.StyleType = field(default="bold green")
    style_metavar: rich.style.StyleType = field(default="bold yellow")
    style_metavar_append: rich.style.StyleType = field(default="dim yellow")
    style_metavar_separator: rich.style.StyleType = field(default="dim")
    style_header_text: rich.style.StyleType = field(default="")
    style_footer_text: rich.style.StyleType = field(default="")
    style_usage: rich.style.StyleType = field(default="yellow")
    style_usage_command: rich.style.StyleType = field(default="bold")
    style_deprecated: rich.style.StyleType = field(default="red")
    style_helptext_first_line: rich.style.StyleType = field(default="")
    style_helptext: rich.style.StyleType = field(default="dim")
    style_option_help: rich.style.StyleType = field(default="")
    style_option_default: rich.style.StyleType = field(default="dim")
    style_option_envvar: rich.style.StyleType = field(default="dim yellow")
    style_required_short: rich.style.StyleType = field(default="red")
    style_required_long: rich.style.StyleType = field(default="dim red")
    style_options_panel_border: rich.style.StyleType = field(default="dim")
    align_options_panel: rich.align.AlignMethod = field(default="left")
    style_options_table_show_lines: bool = field(default=False)
    style_options_table_leading: int = field(default=0)
    style_options_table_pad_edge: bool = field(default=False)
    style_options_table_padding: rich.padding.PaddingDimensions = field(default_factory=lambda: (0, 1))
    style_options_table_box: rich.style.StyleType = field(default="")
    style_options_table_row_styles: Optional[List[rich.style.StyleType]] = field(default=None)
    style_options_table_border_style: Optional[rich.style.StyleType] = field(default=None)
    style_commands_panel_border: rich.style.StyleType = field(default="dim")
    align_commands_panel: rich.align.AlignMethod = field(default="left")
    style_commands_table_show_lines: bool = field(default=False)
    style_commands_table_leading: int = field(default=0)
    style_commands_table_pad_edge: bool = field(default=False)
    style_commands_table_padding: rich.padding.PaddingDimensions = field(default_factory=lambda: (0, 1))
    style_commands_table_box: rich.style.StyleType = field(default="")
    style_commands_table_row_styles: Optional[List[rich.style.StyleType]] = field(default=None)
    style_commands_table_border_style: Optional[rich.style.StyleType] = field(default=None)
    style_commands_table_column_width_ratio: Optional[Union[Tuple[None, None], Tuple[int, int]]] = field(
        default_factory=lambda: (None, None)
    )
    style_errors_panel_border: rich.style.StyleType = field(default="red")
    align_errors_panel: rich.align.AlignMethod = field(default="left")
    style_errors_suggestion: rich.style.StyleType = field(default="dim")
    style_errors_suggestion_command: rich.style.StyleType = field(default="blue")
    style_aborted: rich.style.StyleType = field(default="red")
    width: Optional[int] = field(default_factory=terminal_width_default)
    max_width: Optional[int] = field(default_factory=terminal_width_default)
    color_system: Optional[Literal["auto", "standard", "256", "truecolor", "windows"]] = field(default="auto")
    force_terminal: Optional[bool] = field(default_factory=force_terminal_default)

    # Fixed strings
    header_text: Optional[str] = field(default=None)
    footer_text: Optional[str] = field(default=None)
    deprecated_string: str = field(default="(Deprecated) ")
    default_string: str = field(default="[default: {}]")
    envvar_string: str = field(default="[env var: {}]")
    required_short_string: str = field(default="*")
    required_long_string: str = field(default="[required]")
    range_string: str = field(default=" [{}]")
    append_metavars_help_string: str = field(default="({})")
    arguments_panel_title: str = field(default="Arguments")
    options_panel_title: str = field(default="Options")
    commands_panel_title: str = field(default="Commands")
    errors_panel_title: str = field(default="Error")
    errors_suggestion: Optional[str] = field(default=None)
    """Defaults to Try 'cmd -h' for help. Set to False to disable."""
    errors_epilogue: Optional[str] = field(default=None)
    aborted_text: str = field(default="Aborted.")

    # Behaviours
    show_arguments: bool = field(default=False)
    """Show positional arguments"""
    show_metavars_column: bool = field(default=True)
    """Show a column with the option metavar (eg. INTEGER)"""
    append_metavars_help: bool = field(default=False)
    """Append metavar (eg. [TEXT]) after the help text"""
    group_arguments_options: bool = field(default=False)
    """Show arguments with options instead of in own panel"""
    option_envvar_first: bool = field(default=False)
    """Show env vars before option help text instead of after"""
    use_markdown: bool = field(default=False)
    use_markdown_emoji: bool = field(default=True)
    """Parse emoji codes in markdown :smile:"""
    use_rich_markup: bool = field(default=False)
    """Parse help strings for rich markup (eg. [red]my text[/])"""
    command_groups: Dict[str, List[Dict[str, Union[str, Any]]]] = field(default_factory=lambda: {})
    """Define sorted groups of panels to display subcommands"""
    option_groups: Dict[str, List[Dict[str, Union[str, Any]]]] = field(default_factory=lambda: {})
    """Define sorted groups of panels to display options and arguments"""
    use_click_short_help: bool = field(default=False)
    """Use click's default function to truncate help text"""
    highlighter: rich.highlighter.Highlighter = field(default_factory=lambda: OptionHighlighter())
    """Rich regex highlighter for help highlighting"""
    legacy_windows: Optional[bool] = field(default=False)



"""
rich-click is a minimal Python module to combine the efforts of the excellent packages 'rich' and 'click'.

The intention is to provide attractive help output from click, formatted with rich, with minimal
customisation required.
"""

from typing import Any, Callable, cast, Dict, Optional, overload, Type, TYPE_CHECKING, TypeVar, Union

from click import Command
from click import command as click_command
from click import Group
from click import group as click_group
from click import pass_context as click_pass_context
from rich.console import Console
from typing_extensions import Concatenate, ParamSpec

from . import rich_click  # noqa: F401

from rich_click._compat_click import CLICK_IS_BEFORE_VERSION_8X
from rich_click.rich_command import RichCommand, RichGroup, RichMultiCommand  # noqa: F401
from rich_click.rich_context import RichContext
from rich_click.rich_help_configuration import RichHelpConfiguration

# MyPy does not like star imports. Therefore when we are type checking, we import each individual module
# from click here. This way MyPy will recognize the import and not throw any errors. Furthermore, because of
# the TYPE_CHECKING check, it does not influence the start routine at all.

_AnyCallable = Callable[..., Any]
F = TypeVar("F", bound=Callable[..., Any])
FC = TypeVar("FC", bound=Union[Command, _AnyCallable])


GrpType = TypeVar("GrpType", bound=Group)


# variant: no call, directly as decorator for a function.
@overload
def group(name: _AnyCallable) -> RichGroup:
    ...


# variant: with positional name and with positional or keyword cls argument:
# @group(namearg, GroupCls, ...) or @group(namearg, cls=GroupCls, ...)
@overload
def group(
    name: Optional[str],
    cls: Type[GrpType],
    **attrs: Any,
) -> Callable[[_AnyCallable], GrpType]:
    ...


# variant: name omitted, cls _must_ be a keyword argument, @group(cmd=GroupCls, ...)
@overload
def group(
    name: None = None,
    *,
    cls: Type[GrpType],
    **attrs: Any,
) -> Callable[[_AnyCallable], GrpType]:
    ...


# variant: with optional string name, no cls argument provided.
@overload
def group(name: Optional[str] = ..., cls: None = None, **attrs: Any) -> Callable[[_AnyCallable], RichGroup]:
    ...


def group(
    name: Union[str, _AnyCallable, None] = None,
    cls: Optional[Type[GrpType]] = None,
    **attrs: Any,
) -> Union[Group, Callable[[_AnyCallable], Union[RichGroup, GrpType]]]:
    """
    Group decorator function.

    Defines the group() function so that it uses the RichGroup class by default.
    """
    if cls is None:
        cls = cast(Type[GrpType], RichGroup)

    if callable(name):
        return command(cls=cls, **attrs)(name)

    return command(name, cls, **attrs)


CmdType = TypeVar("CmdType", bound=Command)


# variant: no call, directly as decorator for a function.
@overload
def command(name: _AnyCallable) -> RichCommand:
    ...


# variant: with positional name and with positional or keyword cls argument:
# @command(namearg, CommandCls, ...) or @command(namearg, cls=CommandCls, ...)
@overload
def command(
    name: Optional[str],
    cls: Type[CmdType],
    **attrs: Any,
) -> Callable[[_AnyCallable], CmdType]:
    ...


# variant: name omitted, cls _must_ be a keyword argument, @command(cls=CommandCls, ...)
@overload
def command(
    name: None = None,
    *,
    cls: Type[CmdType],
    **attrs: Any,
) -> Callable[[_AnyCallable], CmdType]:
    ...


# variant: with optional string name, no cls argument provided.
@overload
def command(name: Optional[str] = ..., cls: None = None, **attrs: Any) -> Callable[[_AnyCallable], RichCommand]:
    ...


def command(
    name: Union[Optional[str], _AnyCallable] = None,
    cls: Optional[Type[CmdType]] = None,
    **attrs: Any,
) -> Union[Command, Callable[[_AnyCallable], Union[RichCommand, CmdType]]]:
    """
    Command decorator function.

    Defines the command() function so that it uses the RichCommand class by default.
    """
    if cls is None:
        cls = cast(Type[CmdType], RichCommand)

    if callable(name):
        return click_command(cls=cls, **attrs)(name)

    return click_command(name, cls=cls, **attrs)


class NotSupportedError(Exception):
    """Not Supported Error."""

    pass


def rich_config(
    console: Optional[Console] = None, help_config: Optional[RichHelpConfiguration] = None
) -> Callable[[FC], FC]:
    """Use decorator to configure Rich Click settings.

    Args:
        console: A Rich Console that will be accessible from the `RichContext`, `RichCommand`, and `RichGroup` instances
            Defaults to None.
        help_config: Rich help configuration that is used internally to format help messages and exceptions
            Defaults to None.
    """
    if CLICK_IS_BEFORE_VERSION_8X:

        def decorator_with_warning(obj: FC) -> FC:
            import warnings

            warnings.warn(
                "`rich_config()` does not work with versions of click prior to version 8.0.0."
                " Please update to a newer version of click to use this functionality.",
                RuntimeWarning,
            )
            return obj

        return decorator_with_warning

    def decorator(obj: FC) -> FC:
        extra: Dict[str, Any] = {}
        if console is not None:
            extra["rich_console"] = console
        if help_config is not None:
            extra["rich_help_config"] = help_config

        if isinstance(obj, (RichCommand, RichGroup)):
            obj.context_settings.update(extra)
        elif callable(obj) and not isinstance(obj, (Command, Group)):
            if hasattr(obj, "__rich_context_settings__"):
                obj.__rich_context_settings__.update(extra)
            else:
                setattr(obj, "__rich_context_settings__", extra)
        else:
            raise NotSupportedError("`rich_config` requires a `RichCommand` or `RichGroup`. Try using the cls keyword")
        return obj

    return decorator


# Users of rich_click would face issues using mypy with this code,
# if not for wrapping `pass_context` with a new function signature:
#
# @click.command()
# @click.pass_context
# def cli(ctx: click.RichContext) -> None:
#    ...


P = ParamSpec("P")
R = TypeVar("R")


def pass_context(f: Callable[Concatenate[RichContext, P], R]) -> Callable[P, R]:
    # flake8: noqa: D400,D401
    """Marks a callback as wanting to receive the current context
    object as first argument.
    """
    return click_pass_context(f)  # type: ignore[arg-type]



from setuptools import setup

setup(
    install_requires=["click>=7", "rich>=10.7.0", "importlib-metadata; python_version < '3.8'", "typing_extensions"],
    extras_require={
        "dev": [
            "mypy",
            "pre-commit",
            "pytest",
            "flake8",
            "flake8-docstrings",
            "pytest-cov",
            "packaging",
            "types-setuptools",
        ],
    },
)



import errno
import os
import sys
import warnings
from functools import wraps
from typing import Any, Callable, cast, Optional, overload, Sequence, TextIO, Type, TYPE_CHECKING, Union

import click

# Group, Command, and CommandCollection need to be imported directly,
# or else rich_click.cli.patch() causes a recursion error.
from click import Command, CommandCollection, Group
from click.utils import make_str, PacifyFlushWrapper
from rich.console import Console

from rich_click._compat_click import CLICK_IS_BEFORE_VERSION_8X, CLICK_IS_BEFORE_VERSION_9X
from rich_click.rich_click import rich_abort_error, rich_format_error, rich_format_help
from rich_click.rich_context import RichContext
from rich_click.rich_help_configuration import RichHelpConfiguration
from rich_click.rich_help_formatter import RichHelpFormatter


class RichCommand(click.Command):
    """Richly formatted click Command.

    Inherits click.Command and overrides help and error methods
    to print richly formatted output.

    This class can be used as a mixin for other click command objects.
    """

    context_class: Type[RichContext] = RichContext
    _formatter: Optional[RichHelpFormatter] = None

    @wraps(Command.__init__)
    def __init__(self, *args: Any, **kwargs: Any):
        """Create Rich Command instance."""
        super().__init__(*args, **kwargs)
        self._register_rich_context_settings_from_callback()

    def _register_rich_context_settings_from_callback(self) -> None:
        if self.callback is not None:
            if hasattr(self.callback, "__rich_context_settings__"):
                rich_context_settings = getattr(self.callback, "__rich_context_settings__", {})
                for k, v in rich_context_settings.items():
                    self.context_settings.setdefault(k, v)
                del self.callback.__rich_context_settings__

    @property
    def console(self) -> Optional[Console]:
        """Rich Console.

        This is a separate instance from the help formatter that allows full control of the
        console configuration.

        See `rich_config` decorator for how to apply the settings.
        """
        return self.context_settings.get("rich_console")

    @property
    def help_config(self) -> Optional[RichHelpConfiguration]:
        """Rich Help Configuration."""
        return self.context_settings.get("rich_help_config")

    @property
    def formatter(self) -> RichHelpFormatter:
        """Rich Help Formatter.

        This is separate instance from the formatter used to display help,
        but is created from the same `RichHelpConfiguration`. Currently only used
        for error reporting.
        """
        if self._formatter is None:
            self._formatter = RichHelpFormatter(config=self.help_config)
        return self._formatter

    def main(
        self,
        args: Optional[Sequence[str]] = None,
        prog_name: Optional[str] = None,
        complete_var: Optional[str] = None,
        standalone_mode: bool = True,
        windows_expand_args: bool = True,
        **extra: Any,
    ) -> Any:
        # It's not feasible to use super().main() in this context and retain exact parity in behavior.
        # The reason why is explained in a comment in click's source code in the "except Exit as e" block.

        if args is None:
            if CLICK_IS_BEFORE_VERSION_8X:
                from click.utils import get_os_args  # type: ignore[attr-defined]

                args: Sequence[str] = get_os_args()  # type: ignore[no-redef]
            else:
                args = sys.argv[1:]

                if os.name == "nt" and windows_expand_args:
                    from click.utils import _expand_args

                    args = _expand_args(args)
        else:
            args = list(args)

        if TYPE_CHECKING:
            assert args is not None

        if prog_name is None:
            if CLICK_IS_BEFORE_VERSION_8X:
                prog_name = make_str(os.path.basename(sys.argv[0] if sys.argv else __file__))
            else:
                from click.utils import _detect_program_name

                prog_name = _detect_program_name()

        # Process shell completion requests and exit early.
        if CLICK_IS_BEFORE_VERSION_8X:
            from click.core import _bashcomplete  # type: ignore[attr-defined]

            _bashcomplete(self, prog_name, complete_var)
        else:
            self._main_shell_completion(extra, prog_name, complete_var)

        try:
            try:
                with self.make_context(prog_name, args, **extra) as ctx:
                    rv = self.invoke(ctx)
                    if not standalone_mode:
                        return rv
                    # it's not safe to `ctx.exit(rv)` here!
                    # note that `rv` may actually contain data like "1" which
                    # has obvious effects
                    # more subtle case: `rv=[None, None]` can come out of
                    # chained commands which all returned `None` -- so it's not
                    # even always obvious that `rv` indicates success/failure
                    # by its truthiness/falsiness
                    ctx.exit()
            except (EOFError, KeyboardInterrupt):
                click.echo(file=sys.stderr)
                raise click.exceptions.Abort() from None
            except click.exceptions.ClickException as e:
                rich_format_error(e, self.formatter)
                if not standalone_mode:
                    raise
                sys.stderr.write(self.formatter.getvalue())
                sys.exit(e.exit_code)
            except OSError as e:
                if e.errno == errno.EPIPE:
                    sys.stdout = cast(TextIO, PacifyFlushWrapper(sys.stdout))
                    sys.stderr = cast(TextIO, PacifyFlushWrapper(sys.stderr))
                    sys.exit(1)
                else:
                    raise
        except click.exceptions.Exit as e:
            if standalone_mode:
                sys.exit(e.exit_code)
            else:
                return e.exit_code
        except click.exceptions.Abort:
            rich_abort_error(self.formatter)
            if not standalone_mode:
                raise
            sys.stderr.write(self.formatter.getvalue())
            sys.exit(1)

    def format_help(self, ctx: click.Context, formatter: click.HelpFormatter) -> None:
        rich_format_help(self, ctx, formatter)


class RichGroup(RichCommand, Group):
    """Richly formatted click Group.

    Inherits click.Group and overrides help and error methods
    to print richly formatted output.
    """

    command_class: Optional[Type[RichCommand]] = RichCommand
    group_class: Optional[Union[Type[Group], Type[type]]] = type

    @wraps(Group.__init__)
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize RichGroup class."""
        Group.__init__(self, *args, **kwargs)
        self._register_rich_context_settings_from_callback()

    if CLICK_IS_BEFORE_VERSION_8X:

        @overload
        def command(self, __func: Callable[..., Any]) -> Command:
            ...

        @overload
        def command(self, *args: Any, **kwargs: Any) -> Callable[[Callable[..., Any]], Command]:
            ...

        def command(self, *args: Any, **kwargs: Any) -> Union[Callable[[Callable[..., Any]], Command], Command]:
            # This method override is required for Click 7.x compatibility.
            # (The command_class ClassVar was not added until 8.0.)
            if self.command_class is not None:
                kwargs.setdefault("cls", self.command_class)
            return super().command(*args, **kwargs)  # type: ignore[no-any-return]

        @overload
        def group(self, __func: Callable[..., Any]) -> Group:
            ...

        @overload
        def group(self, *args: Any, **kwargs: Any) -> Callable[[Callable[..., Any]], Group]:
            ...

        def group(self, *args: Any, **kwargs: Any) -> Union[Callable[[Callable[..., Any]], Group], Group]:
            # This method override is required for Click 7.x compatibility.
            # (The group_class ClassVar was not added until 8.0.)
            if self.group_class is type:
                kwargs.setdefault("cls", self.__class__)
            elif self.group_class is not None:
                kwargs.setdefault("cls", self.group_class)
            return super().group(*args, **kwargs)  # type: ignore[no-any-return]


if CLICK_IS_BEFORE_VERSION_9X:
    with warnings.catch_warnings():
        warnings.filterwarnings("ignore", category=DeprecationWarning, module="click")
        from click import MultiCommand

        class RichMultiCommand(RichCommand, MultiCommand):
            """Richly formatted click MultiCommand.

            Inherits click.MultiCommand and overrides help and error methods
            to print richly formatted output.
            """

        @wraps(MultiCommand.__init__)
        def __init__(self, *args: Any, **kwargs: Any) -> None:  # type: ignore[no-untyped-def]
            """Initialize RichGroup class."""
            from click import MultiCommand

            MultiCommand.__init__(self, *args, **kwargs)
            self._register_rich_context_settings_from_callback()

else:

    class RichMultiCommand(RichGroup):  # type: ignore[no-redef]
        """This class is deprecated."""


class RichCommandCollection(RichGroup, CommandCollection):
    """Richly formatted click CommandCollection.

    Inherits click.CommandCollection and overrides help and error methods
    to print richly formatted output.
    """

    @wraps(CommandCollection.__init__)
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize RichCommandCollection class."""
        CommandCollection.__init__(self, *args, **kwargs)
        self._register_rich_context_settings_from_callback()
